@@ -8,10 +8,11 @@ program so
   real, allocatable :: x(:), y(:)
   real, allocatable :: rho(:,:)
   real, allocatable :: psi(:,:)
-  integer :: tp, method
+  integer :: method
   integer :: IWS
   real :: cmin, cmax, contmin, contmax
   character(6) :: form_types, form_typec
+  character(4) :: tp
 
  namelist /input /nx,ny,tp,cmin,cmax,contmin,contmax,IWS,form_types,form_typec,method
  read(5,input)
@@ -47,9 +48,9 @@ program so
  
   select case (method)
   case(1)
-     call Poisson_GauSei(nx,ny,dx,dy,rho,1.0e-6,tp,psi)
+     call Poisson_GauSei(x,y,rho,1.0e-6,tp,psi)
   case(2)
-     call Poisson_Jacobi(nx,ny,dx,dy,rho,1.0e-6,tp,psi)
+     call Poisson_Jacobi(x,y,rho,1.0e-6,tp,psi)
   end select
 
   call color_setting( 10, cmin, cmax )
