@@ -23,27 +23,112 @@ subroutine rectangle_int( x, y, bot, top, res, undeff )  ! 1 ʬ
   real, intent(in) :: y(size(x))  ! ʬؿ
   real, intent(inout) :: res  ! ʬʬ
   real, intent(in), optional :: undeff
-  integer :: i, j, k, nx
-  real :: dx
+  integer :: i, j, k, nx, i_bot, i_top
+  real :: dx, y_bot, y_top
 
   nx=size(x)
 
   res=0.0
 
-  if(present(undeff))then
-     do i=1,nx-1
-        if(x(i)>=bot.and.x(i)<=top)then  ! ʬϰ֤
+!-- bot < top ǤʤХ顼
+  if(bot>top)then
+     write(*,*) "#### ERROR ####"
+     write(*,*) "integrated interval must be bot < top. STOP"
+     stop
+  end if
+
+!-- ʲʬ
+!-- ºݤˤ, ʬ˺Ƕ᤹
+  do i=1,nx
+     if(x(i)>bot)then
+        if(i==1)then
+           write(*,*) "#### WARNING ####"
+           write(*,*) "there is NOT the bot in the x(i)."  ! ΤȤ, i_bot=1ȤƤƤ, x(1)=bot ʤΤ, ;ʬ׻ϥȤʤƶϤʤ.
+        end if
+
+        i_bot=i
+        exit
+
+     end if
+  end do
+
+  do i=1,nx
+     if(x(i)<top)then
+        if(i==nx)then
+           write(*,*) "#### WARNING ####"
+           write(*,*) "there is NOT the top in the x(i)."  ! ΤȤ, i_top=nx ȤƤ, x(nx)=top ʤΤ, ;ʬ׻ϥȤʤ.
+        end if
+
+        i_top=i
+
+     end if
+  end do
+
+!-- ʲǳʻҤƤϤޤʤʬޤ䴰
+  y_bot=y(i_bot-1)  &
+  &     +((y(i_bot)-y(i_bot-1))/(x(i_bot)-x(i_bot-1)))*(bot-x(i_bot-1))
+  y_top=y(i_top)  &
+  &     +((y(i_top+1)-y(i_top))/(x(i_top+1)-x(i_top)))*(x(i_top+1)-top)
+
+  if(i_bot<=i_top)then  ! ʬ˳ʻҤ 1 İʾ夢Ȥ
+
+     if(present(undeff))then
+        do i=i_bot,i_top
            if(y(i+1)/=undeff.and.y(i)/=undeff)then
-              res=res+0.5*(x(i+1)-x(i))*(y(i+1)+y(i))
+              if(i==i_bot)then
+                 if(i_bot<i_top)then  ! ʬ֤ 2 ʻҰʾ夢
+                    res=res+0.5*(x(i)-bot)*(y(i)+y_bot)  &
+  &                     +0.5*(x(i+1)-x(i))*(y(i+1)+y(i))
+                        ! ü;̾ûʬ
+                 else
+                    if(i_bot==i_top)then
+                      ! ʬ֤ 1 ʻҤʤ, ʬԤä, exit ȴ.
+                       res=res+0.5*(x(i)-bot)*(y(i)+y_bot)  &
+  &                        +0.5*(top-x(i))*(y_top+y(i))
+                       exit
+                    end if
+                 end if
+              else
+                 if(i==i_top)then
+                    res=res+0.5*(top-x(i))*(y_top+y(i))  ! ü;Τ
+                 else
+                    res=res+0.5*(x(i+1)-x(i))*(y(i+1)+y(i))  ! ̾û
+                 end if
+              end if
            end if
-        end if
-     end do
+        end do
+     else
+        do i=i_bot,i_top
+           if(i==i_bot)then
+              if(i_bot<i_top)then  ! ʬ֤ 2 ʻҰʾ夢
+                 res=res+0.5*(x(i)-bot)*(y(i)+y_bot)  &
+  &                  +0.5*(x(i+1)-x(i))*(y(i+1)+y(i))
+                     ! ü;̾ûʬ
+              else
+                 if(i_bot==i_top)then
+                   ! ʬ֤ 1 ʻҤʤ, ʬԤä, exit ȴ.
+                    res=res+0.5*(x(i)-bot)*(y(i)+y_bot)  &
+  &                     +0.5*(top-x(i))*(y_top+y(i))
+                    exit
+                 end if
+              end if
+           else
+              if(i==i_top)then
+                 res=res+0.5*(top-x(i))*(y_top+y(i))  ! ü;Τ
+              else
+                 res=res+0.5*(x(i+1)-x(i))*(y(i+1)+y(i))  ! ̾û
+              end if
+           end if
+        end do
+     end if
   else
-     do i=1,nx-1
-        if(x(i)>=bot.and.x(i)<=top)then  ! ʬϰ֤
-           res=res+0.5*(x(i+1)-x(i))*(y(i+1)+y(i))
+     if(present(undeff))then
+        if(y(i_top)/=undeff.and.y(i_bot)/=undeff)then
+           res=0.5*(top-bot)*(y_top+y_bot)
         end if
-     end do
+     else
+        res=0.5*(top-bot)*(y_top+y_bot)
+     end if
   end if
 
 end subroutine rectangle_int
