@@ -59,7 +59,7 @@ end subroutine write_file
 !-----------------------------------------------
 !-----------------------------------------------
 
-subroutine auto_read_file( file_name, d2n, d3n, d2val, d3val, ad2val, ad3val, nx, ny, nz, d2var, d3var, specnz )
+subroutine auto_read_file( file_name, d2n, d3n, d2val, d3val, ad2val, ad3val, nx, ny, nz, d2var, d3var, specnz, intvnz )
 ! ɤ߹ѿθĿꤹ뤳Ȥ, פѿǡϥɤ߹ޤʤ
 ! 롼. (For CReSS)
 ! Υ롼ƤӽФɬ, val_counter ѰդѿƤ.
@@ -76,6 +76,9 @@ subroutine auto_read_file( file_name, d2n, d3n, d2val, d3val, ad2val, ad3val, nx
   integer, intent(in) :: nz  ! z ǿ
   integer, intent(in), optional :: specnz  ! nz ˤĤƻ٤Τ߼Ф
                       ! ξ, ɬ nz = 1 ǤʤФʤʤ.
+  integer, intent(in), dimension(2), optional :: intvnz
+                      ! nz ˤĤ intvnz(1)  intvnz(2) ޤǤι٤Ф.
+                      ! ξ, nz ϤѿǿǤʤФʤʤ.
   real, dimension(nx,ny,d2n), intent(inout) :: d2var  ! d2 ѿ
   real, dimension(nx,ny,nz,d3n), intent(inout) :: d3var  ! d3 ѿ
   integer :: d2m, d3m, d2am, d3am, i, j, k, all_count, d2_count, d3_count
@@ -120,9 +123,15 @@ write(*,*) "len=", d2m, d3m, d2am, d3am
            if(present(specnz))then
               call read_file( file_name, nx, ny, all_count+specnz, d3var(:,:,specnz,j) )
            else
-              do k=1,nz
-                  call read_file( file_name, nx, ny, all_count+k, d3var(:,:,k,j) )
-              end do
+              if(present(intvnz))then
+                 do k=intvnz(1),intvnz(2)
+                    call read_file( file_name, nx, ny, all_count+k, d3var(:,:,k,j) )
+                 end do
+              else
+                 do k=1,nz
+                    call read_file( file_name, nx, ny, all_count+k, d3var(:,:,k,j) )
+                 end do
+              end if
            end if
         end if
      end if
@@ -160,9 +169,19 @@ write(*,*) "len=", d2m, d3m, d2am, d3am
            write(*,*) "read array exceeds d3n. STOP."
            stop
         else
-           do k=1,nz
-              call read_file( file_name, nx, ny, all_count+k, d3var(:,:,k,j))
-           end do
+           if(present(specnz))then
+              call read_file( file_name, nx, ny, all_count+specnz, d3var(:,:,specnz,j) )
+           else
+              if(present(intvnz))then
+                 do k=intvnz(1),intvnz(2)
+                    call read_file( file_name, nx, ny, all_count+k, d3var(:,:,k,j) )
+                 end do
+              else
+                 do k=1,nz
+                    call read_file( file_name, nx, ny, all_count+k, d3var(:,:,k,j) )
+                 end do
+              end if
+           end if
         end if
      end if
      all_count=all_count+nz  ! if ʸɤǤʤƤ nz ʬ
