@@ -104,10 +104,11 @@ subroutine cm_horizon( z, z0m, coem, richard )
 
 end subroutine
 
+!--------------------------------------------------------
 !--------------------------------------------------------
 
-subroutine ust_horizon( cmd, va, velst )
-! ໤® u_* ׻롼
+subroutine cmdva_2_ust_horizon( cmd, va, velst )
+! Х륯, ®٤໤® u_* ׻롼
   use Thermo_Advanced_Function
   implicit none
   real, intent(in), dimension(:,:) :: cmd  !  za ǤΥХ륯
@@ -115,11 +116,42 @@ subroutine ust_horizon( cmd, va, velst )
   real, intent(inout), dimension(size(cmd,1),size(cmd,2)) :: velst  ! ໤® [m/s]
   integer :: i, j, nx, ny
 
+  nx=size(cmd,1)
+  ny=size(cmd,2)
+
 !$omp parallel default(shared)
 !$omp do private(i,j)
      do j=1,ny
         do i=1,nx
-           velst(i,j)=ust( cmd(i,j), va(i,j) )
+           velst(i,j)=cmdva_2_ust( cmd(i,j), va(i,j) )
+        end do
+     end do
+!$omp end do
+!$omp end parallel
+
+end subroutine
+
+!--------------------------------------------------------
+!--------------------------------------------------------
+
+subroutine taurho_2_ust_horizon( taux, tauy, rho, velst )
+! ϤΥǥȿʿ 2 ʬȤι٤Ǥ̩٤໤® u_* ׻롼
+  use Thermo_Advanced_Function
+  implicit none
+  real, intent(in), dimension(:,:) :: taux  ! ٤ǤϤΥǥ x ʬ [N/m]
+  real, intent(in), dimension(size(taux,1),size(taux,2)) :: tauy  ! ٤ǤϤΥǥ y ʬ [N/m]
+  real, intent(in), dimension(size(taux,1),size(taux,2)) :: rho  ! ٤Ǥ̩ [kg/m^3]
+  real, intent(inout), dimension(size(taux,1),size(taux,2)) :: velst  ! ໤® [m/s]
+  integer :: i, j, nx, ny
+
+  nx=size(taux,1)
+  ny=size(taux,2)
+
+!$omp parallel default(shared)
+!$omp do private(i,j)
+     do j=1,ny
+        do i=1,nx
+           velst(i,j)=taurho_2_ust( (/ taux(i,j), tauy(i,j) /), rho(i,j) )
         end do
      end do
 !$omp end do
@@ -270,7 +302,7 @@ end subroutine
 !--------------------------------------------------------
 !--------------------------------------------------------
 
-subroutine Ertel_PV( x, y, z, u, v, w, rho, pt, pv, undeff )
+subroutine Ertel_PV( x, y, z, u, v, w, rho, pt, cor, pv, undeff, hx, hy, hz )
 ! ƥΥݥƥ󥷥뱲٤׻
   use Thermo_Function
   use Thermo_Routine
@@ -284,8 +316,12 @@ subroutine Ertel_PV( x, y, z, u, v, w, rho, pt, pv, undeff )
   real, intent(in) :: w(size(x),size(y),size(z))  ! ®پ z ʬ [m/s]
   real, intent(in) :: rho(size(x),size(y),size(z))  ! ̩پ [kg/m^3]
   real, intent(in) :: pt(size(x),size(y),size(z))  ! ̾ [K]
+  real, intent(in) :: cor(size(x),size(y))  ! ꥪѥ᡼ [/s]
   real, intent(inout) :: pv(size(x),size(y),size(z))  ! PV [Km^2/kgs]
   real, intent(in), optional :: undeff
+  real, intent(in), optional :: hx(size(x),size(y),size(z))  ! 󥰥ե
+  real, intent(in), optional :: hy(size(x),size(y),size(z))  ! 󥰥ե
+  real, intent(in), optional :: hz(size(x),size(y),size(z))  ! 󥰥ե
   real :: tmp1(size(x),size(y),size(z))
   real :: tmp2(size(x),size(y),size(z))
   real :: tmp3(size(x),size(y),size(z))
@@ -297,29 +333,52 @@ subroutine Ertel_PV( x, y, z, u, v, w, rho, pt, pv, undeff )
   integer :: nx  !  1 ǿ
   integer :: ny  !  2 ǿ
   integer :: nz  !  3 ǿ
-  real :: dx  ! x γʻҴֳ [m]
-  real :: dy  ! y γʻҴֳ [m]
-  real :: dz  ! z γʻҴֳ [m]
+  real :: scalex(size(x),size(y),size(z))
+  real :: scaley(size(x),size(y),size(z))
+  real :: scalez(size(x),size(y),size(z))
 
   nx=size(x)
   ny=size(y)
   nz=size(z)
-  dx=x(2)-x(1)
-  dy=y(2)-y(1)
-  dz=z(2)-z(1)
+
+  if(present(hx).and.present(hy).and.present(hz))then
+     do k=1,nz
+        do j=1,ny
+           do i=1,nx
+              scalex(i,j,k)=hx(i,j,k)
+              scaley(i,j,k)=hy(i,j,k)
+              scalez(i,j,k)=hz(i,j,k)
+           end do
+        end do
+     end do
+  else
+     do k=1,nz
+        do j=1,ny
+           do i=1,nx
+              scalex(i,j,k)=1.0
+              scaley(i,j,k)=1.0
+              scalez(i,j,k)=1.0
+           end do
+        end do
+     end do
+  end if
 
   if(present(undeff))then
 !  ̤ζָۤ׻.
-     call grad_vec_3d( x, y, z, pt, tmp1, tmp2, tmp3, undeff )
+     call grad_vec_3d( x, y, z, pt, tmp1, tmp2, tmp3, undeff,  &
+  &                    scalex, scaley, scalez )
 !  3  rotation ׻.
-     call rotate( x, y, z, u, v, w, tmp4, tmp5, tmp6, undeff )
+     call rotate( x, y, z, u, v, w, tmp4, tmp5, tmp6, undeff,  & 
+  &               scalex, scaley, scalez )
 !  omega  grad pt Ѥ׻
      call dot_prod( tmp4, tmp5, tmp6, tmp1, tmp2, tmp3, tmp7, undeff )
   else
 !  ̤ζָۤ׻.
-     call grad_vec_3d( x, y, z, pt, tmp1, tmp2, tmp3 )
+     call grad_vec_3d( x, y, z, pt, tmp1, tmp2, tmp3,  &
+  &                    hx=scalex, hy=scaley, hz=scalez )
 !  3  rotation ׻.
-     call rotate( x, y, z, u, v, w, tmp4, tmp5, tmp6 )
+     call rotate( x, y, z, u, v, w, tmp4, tmp5, tmp6,  & 
+  &               hx=scalex, hy=scaley, hz=scalez )
 !  omega  grad pt Ѥ׻
      call dot_prod( tmp4, tmp5, tmp6, tmp1, tmp2, tmp3, tmp7 )
   end if
@@ -334,7 +393,8 @@ subroutine Ertel_PV( x, y, z, u, v, w, rho, pt, pv, undeff )
               if(tmp7(i,j,k)==undeff.or.rho(i,j,k)==undeff)then
                  pv(i,j,k)=undeff
               else
-                 pv(i,j,k)=tmp7(i,j,k)/rho(i,j,k)
+                 pv(i,j,k)=(tmp7(i,j,k)+cor(i,j)*tmp3(i,j,k))/rho(i,j,k)
+                 ! ׻Τб٤ʤΤ, ꥪ­碌.
               end if
            end do
         end do
@@ -351,7 +411,7 @@ subroutine Ertel_PV( x, y, z, u, v, w, rho, pt, pv, undeff )
      do k=1,nz
         do j=1,ny
            do i=1,nx
-              pv(i,j,k)=tmp7(i,j,k)/rho(i,j,k)
+              pv(i,j,k)=(tmp7(i,j,k)+cor(i,j)*tmp3(i,j,k))/rho(i,j,k)
            end do
         end do
      end do
@@ -363,6 +423,12 @@ subroutine Ertel_PV( x, y, z, u, v, w, rho, pt, pv, undeff )
 
 end subroutine
 
+!--------------------------------------------------------
+!--------------------------------------------------------
+
+!--------------------------------------------------------
+!--------------------------------------------------------
+
 
 
 end module
